'use strict';

/**
 * Created by moyu on 2017/7/30.
 */
var req = require('./lib/require');
req.unover();
req.over();

var express = require('express');
var getPort = require('get-port');
var nps = require('path');
var chokidar = require('chokidar');
var _ = require('lodash');
var inherits = require('util').inherits;
var dirViewMiddleware = require('express-dirview-middleware');
var setWebpackMiddleware = require('./lib/setup-webpack-middleware');
var preInstall = require('./lib/pre-install');
var errorMiddleware = require('./lib/error-middleware');
var ConfigAdaptor = require('./lib/ConfigAdaptor');
var routes = require('./lib/routes');
var EntryHandler = require('./lib/EntryHandler');

function GoJS(opts) {
    this.opts = _.merge({
        verbose: true,
        path: process.cwd(),
        type: 'js',
        port: null
    }, opts);
    if (this.opts.path) {
        this.opts.path = nps.resolve(this.opts.path);
    }

    req.unover();
    req.over([this.opts.path]);

    this._init();
}

GoJS.prototype._init = function () {
    var _this = this;

    this.prevCwd = process.cwd();
    process.chdir(this.opts.path);
    // process.chdir(nps.join(__dirname, '..'))

    // install after chdir
    preInstall(this.opts.type);

    this.entryHandler = new EntryHandler(this.opts.path).init();
    this.configAdaptor = new ConfigAdaptor(this.opts.path, this.opts.type);
    this.configAdaptor.on('addEntry', function (entry) {
        return _this.emit('addEntry', entry);
    });

    this.app = express();
    this.app.locals.opts = this.opts;
    this.app.locals.entryHandler = this.entryHandler;
    this.app.locals.gojs = this;
    this.app.locals.configAdaptor = this.configAdaptor;

    this.app.all('/', function (req, res, next) {
        res.sendFile(nps.join(__dirname, 'template/index.html'));
    });
    this.app.use('/__gojs/file-view/', dirViewMiddleware({ root: this.opts.path, redirect: true }));
    this.app.use(function (req, res, next) {
        var now = Date.now();
        res.on('finish', function () {
            return _this.emit('request', req, res, now);
        });
        next();
    });

    this.app.use(routes);
    this.app.use(express.static(this.opts.path));
    this.app.use(errorMiddleware);

    this.watcher = chokidar.watch(this.opts.path, {
        ignoreInitial: true,
        ignoreCase: /node_modules|\.git|^\..+|[\/\\]\..+/
    });
    this.watcher.on('unlink', function (fp) {
        fp = fp.substring(_this.opts.path.length).replace(/^\//g, '');
        if (/jsx?$/.test(fp) && !/(node_modules)/.test(fp) && !/^\./.test(fp)) {
            _this.emit('watch', 'unlink', fp);

            if (_this.configAdaptor.rmEntry(fp)) {
                _this.emit('rmEntry', fp);

                // setWebpackMiddleware(this.app, this.configAdaptor.getConfig())
            }
        }
    });
};

GoJS.prototype.start = function (cb) {
    var _this2 = this;

    var listenCallBack = function listenCallBack(err) {
        if (err) {
            cb && cb(err);
            _this2.emit('error', err);
        } else {
            _this2.running = true;
            cb && cb(null, _this2.opts.port);
            _this2.emit('server', _this2.opts.port);

            if (_this2.opts.openSync) {
                var path = _this2.opts.initPath || '';
                path = path.replace(/^\/*/, '');

                require('opn')('http://localhost:' + _this2.opts.port + '/' + path);
            }
        }
    };

    if (this.opts.port) {
        this.server = this.app.listen(this.opts.port, listenCallBack);
    } else {
        getPort().then(function (port) {
            _this2.opts.port = port;
            _this2.server = _this2.app.listen(port, listenCallBack);
        }).catch(function (err) {
            cb && cb(err);
            _this2.emit('error', err);
        });
    }
};

GoJS.prototype.stop = function (cb) {
    if (!cb) cb = function cb() {};
    if (!this.running) {
        return cb(new Error('No server currently running.'));
    }

    if (this.running) {
        this.removeAllListeners();
        this.entryHandler.exit();
        this.watcher && this.watcher.close();
        process.chdir(this.prevCwd);
        this.server.close(cb);
        this.running = false;
    }
};

inherits(GoJS, require('events').EventEmitter);

module.exports = GoJS;