'use strict';

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Created by moyu on 2017/7/30.
 */
var nps = require('path');
var _ = require('lodash');
var EventEmitter = require('events').EventEmitter;
var inherits = require('util').inherits;
var log = require('./log');
var encodeSep = require('./encode-decode').encode;

var emptyBuildins = ['child_process', 'cluster', 'dgram', 'dns', 'fs', 'module', 'net', 'readline', 'repl', 'tls'];

function getConfig(root, type, dev) {
    if (typeof dev === 'undefined') dev = true;
    var loaders = null;
    if (!type || type === 'js') {
        try {
            loaders = require(nps.join(root, 'gojs.jsloader.js'));
            log.info('custom js loader worked. ' + nps.join(root, 'gojs.jsloader.js'));
        } catch (e) {
            loaders = require('./loaders/js-loader');
        }
    }

    loaders = loaders.map(function (loader) {
        loader.include = loader.include || [];
        loader.include.push(root);
        return loader;
    });

    var projPath = nps.join(__dirname, '../..');
    // const projPath = root
    return {
        devtool: dev ? 'source-map' : null,
        context: projPath,
        // context: root,
        entry: {},
        output: {
            publicPath: '/__gojs/bundle/',
            path: nps.join(root, '/.dist/'),

            filename: '[name].bundle.js',
            chunkFilename: '[name].[chunkhash].bundle.js'
        },
        module: {
            loaders: loaders
        },
        cache: true,
        node: emptyBuildins.reduce(function (collection, key) {
            collection[key] = 'empty';
            return collection;
        }, {}),

        resolve: {
            // root: nps.join(projPath),
            root: [nps.join(root, 'node_modules'), nps.join(projPath, 'node_modules')]
        },

        resolveLoader: {
            root: [nps.join(root, 'node_modules'), nps.join(projPath, 'node_modules')]
            // root: nps.join(root),
        },

        plugins: []
    };
}

function clearEntry() {
    this.config.entry = {};
    return this;
}

function addEntry(key, abPath) {
    var encodeName = encodeSep(key);
    if (!this.config.entry[encodeName]) {
        abPath = abPath || nps.join(this.root, key);
        var entry = fillEntry(abPath, !this.dev);
        this.config.entry[encodeName] = entry;
        this.emit('addEntry', _defineProperty({}, encodeName, entry));
        return entry.slice();
    }
    return false;
}

function rmEntry(key) {
    var encodeName = encodeSep(key);
    if (this.config.entry[encodeName]) {
        delete this.config.entry[encodeName];
        return true;
    }
    return false;
}

function fillEntry(absolute, prod) {
    if (prod) {
        return ['babel-polyfill',
        // 'react-hot-loader/patch',
        absolute];
    }

    return ['babel-polyfill',
    // 'react-hot-loader/patch',
    'webpack-hot-middleware/client?reload=true', absolute];
}

function ConfigAdaptor(root, type, isDev) {
    if (typeof isDev === 'undefined') isDev = true;
    this.dev = isDev;
    this.config = getConfig(root, type, isDev);
    this.root = root;
}

ConfigAdaptor.prototype.clearEntry = clearEntry;
ConfigAdaptor.prototype.addEntry = addEntry;
ConfigAdaptor.prototype.rmEntry = rmEntry;
ConfigAdaptor.prototype.getConfig = function () {
    return this.config;
};
ConfigAdaptor.fillEntry = fillEntry;

inherits(ConfigAdaptor, EventEmitter);

module.exports = ConfigAdaptor;