'use strict';

/**
 * Created by moyu on 2017/7/30.
 */
var nps = require('path');
var fs = require('fs');
var _ = require('lodash');
var decodeSep = require('./encode-decode').decode;
var setupWebpackMiddleware = require('./setup-webpack-middleware');

var errorStr = fs.readFileSync(nps.join(__dirname, '../template/error.html')).toString();
var renderError = _.template(errorStr);

module.exports = function (err, req, res, next) {
    var path = req.path;

    var locals = req.app.locals;
    locals.gojs.emit('error', err);

    var status = 500;
    var message = err.message;

    if (err.code === 'ENOENT') {
        var pathType = nps.extname(err.path) !== '' ? 'file' : 'folder';
        status = 404;
        message = 'The ' + pathType + ' ' + req.path + ' was not found. Maybe you didn\'t create it yet?';
    }

    if (err.code === 'ENOINDEX') {
        status = 404;
        message = 'The folder ' + req.path + ' doesn\'t have an index file. Try creating an index.js file in this folder.';
    }

    var template = renderError({
        status: status,
        message: message
    });

    res.contentType('html');
    return res.status(status).send(template);
};